/*
 The ECLiPSe interface to MiniSat SAT solver
 http://www.cs.chalmers.se/Cs/Research/FormalMethods/MiniSat/MiniSat.html

 Copyright (c) 2006, Michael Codish, Vitaly Lagoon, and Peter J. Stuckey
 Modified for ECLiPSe by Joachim Schimpf
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


#include "eclipse.h"
#include "solver.h"

#define MAX_CLAUSE_LENGTH 10000


static lit clause_buffer[MAX_CLAUSE_LENGTH];

static inline lit pl2lit(pword pl_literal)
{
    long pl_lit_int;
    ec_get_long(pl_literal,&pl_lit_int);
    return (pl_lit_int<0) ? neg(toLit(-pl_lit_int)) : toLit(pl_lit_int);
}


static void
minisat_free_method(t_ext_ptr s)
{
    solver_delete((solver *) s);
}

t_ext_type minisat_type = {minisat_free_method,0,0,0,0,0,0,0,0};

int minisat_new_solver()
{
    solver *s = solver_new();
    return ec_unify(ec_arg(1), ec_handle(&minisat_type, (t_ext_ptr) s));
}

int minisat_delete_solver()
{
    return ec_free_handle(ec_arg(1), &minisat_type);
}

int minisat_add_clause()
{
    int res;
    int pl_literal;
    pword head;	 		     /* variable for the elements */
    pword list = ec_arg(2);
    int *ptr = clause_buffer;
    bool status;
    solver *s;
    res = ec_get_handle(ec_arg(1), &minisat_type, (t_ext_ptr*) &s);
    if (res != PSUCCEED) return res;
    while((res = ec_get_list(list, &head, &list)) == PSUCCEED) {
	*ptr = pl2lit(head);
	ptr++;
    }
    if (res != PFAIL) return res;
    
    return solver_addclause(s,clause_buffer,ptr) ? PSUCCEED : PFAIL;
}

int minisat_solve()
{
    solver *s;
    int res = ec_get_handle(ec_arg(1), &minisat_type, (t_ext_ptr*) &s);
    if (res != PSUCCEED) return res;

    if (!solver_simplify(s)) return PFAIL;
    if (!solver_solve(s,NULL,NULL)) return PFAIL;

    {
	pword l = ec_nil();
	int pl_lit_int;
	int i = solver_nvars(s);
	void **vals = vec_begin(&(s->model));
	
	while( --i >= 0 ) {
	    switch((int)((lbool)vals[i])) {
	    case  1: pl_lit_int = i;  break;
	    case -1: pl_lit_int = -i; break;
	    case  0: pl_lit_int = 0;
	    default : return RANGE_ERROR;
	    }
	    if (pl_lit_int) {
		l = ec_list(ec_long(pl_lit_int), l);
	    }
	}
	return ec_unify(ec_arg(2),l);
    }
}

